#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import numpy as np
import scipy as spy
import scipy.stats as spystats
import sys
import os,shutil, fnmatch, pickle
from sys import exit
import pdb
import pandas as pd
import glob
import scipy.linalg   # SciPy Linear Algebra Library

from scipy.stats import norm
from scipy.stats import uniform

from mpl_toolkits import mplot3d
import matplotlib.pyplot as plt

from scipy.stats import qmc
import utilRF

listTest=['cellRF','rnnRF']
testNb=1
test= listTest[testNb]
recomputePDF=True #recompute the pdf or laod the file

if test=='cellRF':
  mechDir='../cellRF' # mechanical folder and file
  nDim = 3            # no. dimensions
  lc = [10., 10., 10.]            # correlation length
  # assume X1, X2 to be normally distributed variables
  meanX = np.array([0.05])    # mean of variables
  stdX = np.array([0.01])       # std of variables
  R0_XY = np.array([[1]])    # correlation matrix between parameters C_XY (tau=0)
elif test=='rnnRF':
  mechDir='../rnnRF' # mechanical folder and file
  nDim = 3            # no. dimensions
  lc = [15., 15., 15.]            # correlation length
  # assume X1, X2 to be normally distributed variables
  meanX = np.array([0.5,10])    # mean of variables
  stdX = np.array([0.05,0.1])       # std of variables
  rho = 0.2
  R0_XY = np.array([[1, rho], [rho, 1]])    # correlation matrix between parameters C_XY (tau=0)

Nsim =10 #number of random fields to generate
Gaussian=0
rfDir=mechDir+'/randomFields'
gpInfoDir=mechDir+'/GPData'

######

CHECK_FOLDER = os.path.isdir(rfDir)
if not CHECK_FOLDER:
    os.makedirs(rfDir)
    print("created folder : ", rfDir)

if recomputePDF:
  #=============================================================================
  #extract GP coordinates
  gPCoord,gPTetNb,gPNb,gCoordMin,gCoordMax=utilRF.readGP(gpInfoDir) # x,y,z coordinates = (nGTotal,3),     tetraheadral number of each GP, GP number in each Tet, min coordinates, max coordinates
  print('minimum coordinates of GP: ',gCoordMin)
  print('maximum coordinates of GP: ',gCoordMax)

  # fill all grid points where to generate the RF
  x = [[gPCoord[k][0], gPCoord[k][1], gPCoord[k][2]] for k in range(len(gPCoord))]
  #print(x)


  #=================================
  #compute correlation matrix
  nDtau= 11 
  dtau = [lc[0]/5., lc[1]/5., lc[2]/5.]          # dtau = [dtaux, dtauy, dtauz] used to build the correlation matrix should be lower than correlation length
  ntau = np.array([nDtau, nDtau, nDtau])   # [ntau_x, ntau_y, ntau_z]
  Grille_Dim = dtau               # separation distance (3,)

  # define correlation function with tau = m; l = n
  funccorr = lambda m,n: np.exp(-np.pi*(m**2)/(n**2))

  nVar=len(meanX)
  C0_XY = np.zeros((nVar,nVar))
  for i in range(nVar):
    for j in range(nVar):
        C0_XY[i,j] = R0_XY[i,j]*stdX[i]*stdX[j]
  C_XY = utilRF.RFCorrMat(nVar,nDim,ntau,dtau,funccorr,C0_XY,lc)  # (nVar^2,nx,ny,nz)
  R_rs = C_XY                     # (nVar^2,nx,ny,nz)
  # cross-covariance matrix, R_rs is a 4th-order tensor (nVar^2,ntau1,ntau2,ntau3)
  #print(R_rs)


  #==============================================================================
  ## adding extra discrete points in such a symmetric arrangement
  #==============================================================================
  # check the size of the required random field --> [x1_max, x2_max, x3_max]
             
  # if the original random field is not big enough, add zero at the end of C_XY
  Ori_shape_D0 = R_rs.shape[1:len(R_rs.shape)]    # no. coord. grid points in 3D --> C_XY.shape[1:(nDim+1)]
  list_shape_D = list(Ori_shape_D0)               # no. coord. grid points = [10,10,10]
  for i in range(nDim):                         # no. dimensions
    l_max = Ori_shape_D0[i]*Grille_Dim[i]       # total distance in ith direction
    
    # currently this condition is not activated
    if (l_max < (gCoordMax[i]-gCoordMin[i])):
        list_shape_D[i] = int((gCoordMax[i]-gCoordMin[i])/Grille_Dim[i])+1     # no. original points = list [1]
        print("Correct spectral grid from ",Ori_shape_D0," to ",list_shape_D)

  #==============================================================================
  # adding extra discrete points in such a symmetric arrangement R(-x) = R(x)
  #==============================================================================
  for i in range(len(list_shape_D)):          # no. dimensions
    list_shape_D[i] = 2*list_shape_D[i]-1   # double the no. grid points - 1

  Ori_shape_D = tuple(list_shape_D)  
  R_shape = (R_rs.shape[0],)+Ori_shape_D  
  R_tmp = np.zeros(R_shape)           

  R_rs = utilRF.RFCorrMatExpand(Ori_shape_D,R_rs,R_shape,R_tmp)   # (nVar^2,2ntx-2,2nty-2,2ntz-2)
  print('Grid in freq. domain', Ori_shape_D)
  
  #==============================================================================
  # Spectral Analysis - H_pdf = (18,18,18,4) for 3D
  H_pdf = utilRF.RFSpecAnalysis(R_rs,Grille_Dim)

  #print('H_pdf before save load: ', H_pdf)
  #print('Ori_shape_D before save load: ', Ori_shape_D)
  with open(rfDir+'/H_pdf.dat','wb') as data:
    pickle.dump(H_pdf,data)
    pickle.dump(Ori_shape_D,data)
    pickle.dump(Grille_Dim,data)
  #==============================================================================
  # Save input data of RF grid
  #==============================================================================
  with open(rfDir+'/input.dat','wb') as data:
    pickle.dump(Nsim,data)
    pickle.dump(nVar,data)
    pickle.dump(nDim,data)
    pickle.dump(gPTetNb,data)
    pickle.dump(gPNb,data)
    pickle.dump(x,data)

try:
  with open(rfDir+'/H_pdf.dat','rb') as data:
     H_pdf=pickle.load(data)
     Ori_shape_D=pickle.load(data)
     Grille_Dim=pickle.load(data)
except:
  print('Cannot read HDF file')
  os._exit(1)
else:
  #print('H_pdf after save load: ', H_pdf)
  #print('Ori_shape_D after save load: ', Ori_shape_D)
  with open(rfDir+'/input.dat','rb') as data:
    Nsim=pickle.load(data)
    nVar=pickle.load(data)
    nDim=pickle.load(data)
    gPTetNb=pickle.load(data)
    gPNb=pickle.load(data)
    x=pickle.load(data)

  #==============================================================================
  # Random field
  #==============================================================================
  # output = list [Nsim](nCXYZ,nVar)
  #exit()
  for i in range(Nsim):
    formerH_pdf=H_pdf
    output, H_pdf = utilRF.RFCoeffs(Ori_shape_D,H_pdf,x,Grille_Dim,Gaussian,1)
    H_pdf=formerH_pdf
    # shift the distribution a mean distance
    for k in range(nVar):
      output[0][:,k] = output[0][:,k] + meanX[k]      # (Nsim,nCoord,nVar)

    # writing vector of random field to *.csv files
    rffilename=rfDir+'/RandField_'+str(i)+'.csv'
    print("write RF file ",rffilename)
    utilRF.RFWriteOneRandField(rffilename,output[0],nVar)
    rffilename=rfDir+'/RandField_And_GP_'+str(i)+'.csv'
    print("write RF file ",rffilename)
    utilRF.RFWriteOneGPAndRandField(rffilename,output[0],x,nVar)


